//=============================================================================
//
//  CLASS PinholeCamera - IMPLEMENTATION
//
//=============================================================================
#define RAYTRACER_PINHOLECAMERA_CPP

//== INCLUDES =================================================================
#include "PinholeCamera.h"

//== IMPLEMENTATION ==========================================================
void PinholeCamera::MakeONB()
{
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    VectorType3 w = ( m_v3LookAt - m_v3Eye ).normalize();
    VectorType3 u = ( w % m_v3Up ).normalize();
    VectorType3 v = u % w;
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    m_m33ONB.setRow( 0, u );
    m_m33ONB.setRow( 1, v );
    m_m33ONB.setRow( 2, w );
}

//-----------------------------------------------------------------------------
Ray PinholeCamera::generateRay(
    double    x,      
    double    y,      
    double    rWidth, 
    double    rHeight )  const
{
    /*~~~~*/
    Ray ray;
    /*~~~~*/

    ray.setOrigin( m_v3Eye );

    /*~~~~~~~~~~~~~~~~~~~*/

    //ray-dir in uvw-coordinates (looking down the w-axis)
    VectorType3 v3_uvw_dir;
    /*~~~~~~~~~~~~~~~~~~~*/

    v3_uvw_dir[0] = m_rLeft + ( m_rRight - m_rLeft ) * ( (x + 0.5) / (rWidth - 1) );
    v3_uvw_dir[1] = m_rTop + ( m_rBottom - m_rTop ) * ( (y + 0.5) / (rHeight - 1) );
    v3_uvw_dir[2] = m_rFocalLength;

    //ray-dir in world-coordinates
    ray.setDir( v3_uvw_dir * m_m33ONB );
    return ray;
};
