//=============================================================================
//
//  CLASS Sphere
//
//=============================================================================
#ifndef RAYTRACER_SPHERE_H
#define RAYTRACER_SPHERE_H

//== INCLUDES =================================================================
#include "Surface.h"


//== CLASS DEFINITION =========================================================

/*!
	A sphere. The typical first raytracing-surface!
*/
class Sphere : public Surface
{

    //-------------------------------------------------------------- private data
private:
    VectorType3 m_v3Center;
    double    m_rRadius;

    //-------------------------------------------------------------- private methods
private:
    VectorType2 cartesian2Polar( VectorType3 &v ) const;


public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
    Sphere()
    {
    };

    //! initializing constructor
    Sphere(
        const VectorType3   &v3Center,  
        const double      &rRadius )  :
    m_v3Center( v3Center ),
    m_rRadius( rRadius )
    {
    };

    //-------------------------------------------------------------- element access
    void setCenter(
        const VectorType3   &v3Center ) 
    {
        m_v3Center = v3Center;
    };

    
    const VectorType3 &getCenter() const
    {
        return m_v3Center;
    };

    
    void setRadius(
        const double  &rRadius )  
    {
        m_rRadius = rRadius;
    };

    
    const double getRadius() const
    {
        return m_rRadius;
    };

    
    AABBox getAABB() const
    {
        VectorType3 v3_offset = VectorType3::vectorized( m_rRadius );
        return AABBox( m_v3Center - v3_offset, m_v3Center + v3_offset );
    };

    //render object in OpenGL
    void    renderGL() const;

    //translate object
    void    translate( float x, float y, float z );

    //rotate object
    void    rotate( float rotx, float roty, float rotz );

    //get Translation
    void getTranslation(
        float   *x,     
        float   *y,     
        float   *z )    
    {
        *x = m_v3Center[0];
        *y = m_v3Center[1];
        *z = m_v3Center[2];
    };

    //get Rotation
    void getRotation(
        int *rotx,  
        int *roty,  
        int *rotz ) 
    {
        *rotx = 0;
        *roty = 0;
        *rotz = 0;
    };

    //--------------------------------------------------------------- intersections
    bool    intersect( const Ray       &clRay,
                       const double  t0,
                       const double  t1,
                       TracingContext  &tContext ) const;
};

//=============================================================================
#endif // RAYTRACER_SPHERE_H

