//=============================================================================
//
//  CLASS Surface
//
//=============================================================================
#ifndef RAYTRACER_SURFACE_H
#define RAYTRACER_SURFACE_H

//== INCLUDES =================================================================
#include "Types.h"
#include "AABBox.h"
#include "Ray.h"
#include "Shader.h"
#include <QGLWidget>

//== CLASS DEFINITION =========================================================

/*!
	Abstract base class for all objects, that could be hit by a ray.
*/
class   Surface
{

    //-------------------------------------------------------------- private data
protected:
    const Shader    *m_shader;
    bool            m_selected;

    //-------------------------------------------------------------- private methods
private:

public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
    Surface() :
    m_shader( 0 ),
    m_selected( false )
    {
    };

    //-------------------------------------------------------------- element access
    virtual void setShader(
        const Shader    *pclShader )    
    {
        m_shader = pclShader;
    };

    
    virtual const Shader *getShader() const
    {
        return m_shader;
    };

    //! Return an AABB for this surface.
    virtual AABBox  getAABB() const = 0;

    //! render Surface in OpengL.
    virtual void    renderGL() const = 0;

    //translate Object
    virtual void    translate( float x, float y, float z ) = 0;

    //rotate Object
    virtual void    rotate( float rotx, float roty, float rotz ) = 0;

    //get Translation
    virtual void    getTranslation( float *x, float *y, float *z ) = 0;

    //get Rotation
    virtual void    getRotation( int *rotx, int *roty, int *rotz ) = 0;

    
    virtual void setSelected(
        bool    sel )   
    {
        m_selected = sel;
    };

    
    virtual bool getSelected() const
    {
        return m_selected;
    };

    //--------------------------------------------------------------- intersections
    //! intersection in [t0, t1)?.
    virtual bool    intersect( const Ray       &clRay,
                               const double  t0,
                               const double  t1,
                               TracingContext  &tContext ) const = 0;
};
//=============================================================================
#endif // RAYTRACER_SURFACE_H

